@@ -200,104 +200,132 @@ void bowl_compute_cpu_dest( Bowl *bowl )
 }
 /*
 ====================================================================
-Initate next block.
+Initate next block. Set bowl::block::id to preview and get id of next 
+block to be dealt to bowl::next_block_id.
 ====================================================================
 */
-void bowl_create_next_block( Bowl *bowl ) 
+void bowl_select_next_block( Bowl *bowl ) 
 {
-    int i, min, *new_next_blocks = 0;
-    
-    bowl->block.id = bowl->next_block_id;
-    /* for experts: weight probability of next block against to 
-     * helpfulness with a 50% chance. equal properties else. */
-    if ( config.expert && (rand()%2) ) { 
-        int i, j, threshold, saveblockid; 
-        struct { int block, score; } tmp, scores[BLOCK_COUNT];
+	int i, min, *new_next_blocks = 0;
+	int num_old_blocks = 0;
 
-        saveblockid = bowl->block.id;
-        for (i=0; i<BLOCK_COUNT; i++ ) {
-            bowl->block.id = i;
-            bowl_compute_cpu_dest( bowl );
-            scores[i].block = i;
-            scores[i].score = bowl->cpu_dest_score;
-        }
-        /* Sort */
-        for ( i=0; i<BLOCK_COUNT-1; i++ ) {
-            for ( j=i+1; j<BLOCK_COUNT; j++ ) {
-                if ( scores[j].score < scores[i].score ) {
-                    tmp = scores[i];
-                    scores[i] = scores[j];
-                    scores[j] = tmp;
-                }
-            }
-        }
-        /* 50% chance of worst block, 25% next worse, etc.. */
-        j = rand();
-        threshold = RAND_MAX / 2;
-        for (i=0; i<BLOCK_COUNT-1; i++ ) {
-            if ( j > threshold ) break;
-            threshold /= 2;
-        }
-        bowl->next_block_id = scores[i].block;
-        bowl->block.id = saveblockid;
-    } else {
-        /* Even next-block probabilities */
-        if ( bowl->use_same_blocks )
-        {
-            bowl->next_block_id = next_blocks[bowl->next_blocks_pos++];
-            if ( bowl->next_blocks_pos == next_blocks_size )
-            {
-                /* resize block buffer and get new blocks */
-                min = next_blocks_size;
-                for ( i = 0; i < BOWL_COUNT; i++ )
-                    if ( bowls[i] && bowls[i]->next_blocks_pos < min )
-                        min = bowls[i]->next_blocks_pos;
-                for ( i = 0; i < BOWL_COUNT; i++ )
-                    if ( bowls[i] )
-                        bowls[i]->next_blocks_pos -= min;
-                new_next_blocks = 
-                    calloc( next_blocks_size - min + 
-                            NEXT_BLOCKS_CHUNK_SIZE,
-                            sizeof(int) );
-                memcpy( new_next_blocks, &next_blocks[min],
-                        sizeof(int) * (next_blocks_size - min) );
-                fill_int_array_rand( new_next_blocks, 
-                                     next_blocks_size - min,
-                                     NEXT_BLOCKS_CHUNK_SIZE,
-                                     0, BLOCK_COUNT-1 );
-                free( next_blocks );
-                next_blocks = new_next_blocks;
-                next_blocks_size = next_blocks_size - min +
-                                   NEXT_BLOCKS_CHUNK_SIZE;
-            }
-        }
-        else
-        {
-            bowl->next_block_id = rand() % BLOCK_COUNT; 
-            if ( bowl->next_block_id == bowl->block.id )
-                bowl->next_block_id = rand() % BLOCK_COUNT;
-        }
-    }
-    bowl->block.x = 3;
-    bowl->block.y = -3;
-    bowl->block.sx = bowl->sx + bowl->block_size * bowl->block.x;
-    bowl->block.sy = bowl->sy + bowl->block_size * bowl->block.y;
-    bowl->block.rot_id = 0;
-    bowl->block.sw = bowl->block.sh = 4 * bowl->block_size;
-    bowl->block.cur_x = bowl->block.x * bowl->block_size;
-    bowl->block.cur_y = bowl->block.y * bowl->block_size;
-    bowl->block.check_y = (int)bowl->block.cur_y;
-    bowl_compute_help_pos( bowl );
-    bowl_compute_preview_pos( bowl );
-    /* if CPU is in control get destination row & other stuff */
-    if ( !bowl->controls ) {
-        /* destination */
-        bowl_compute_cpu_dest( bowl );
-        /* set delay until cpu waits with dropping block */
-        delay_set( &bowl->cpu_delay, config.cpu_delay );
-        bowl->cpu_down = 0;
-        delay_set( &bowl->cpu_rot_delay, config.cpu_rot_delay );
-    }
+	/* set preview as current block id */
+	bowl->block.id = bowl->next_block_id;
+	
+	/* in expert mode (only for single player as we will alter the bags!)
+	 * test remaining pieces in current bag and swap worst piece to 
+	 * beginning of queue. */
+	if (config.expert && (config.gametype == 1 || config.gametype == 2)) {
+		int num_test_blocks;
+		int j, savedblockid; 
+		struct { int block, score, next_blocks_pos; } tmp, 
+			scores[BLOCK_COUNT];
+
+		/* determine number of remaining blocks in current bag */
+		num_test_blocks = BLOCK_COUNT - (bowl->next_blocks_pos % BLOCK_COUNT);
+		if (num_test_blocks > next_blocks_size - bowl->next_blocks_pos)
+			num_test_blocks = next_blocks_size - bowl->next_blocks_pos;
+		DPRINTF("Expert mode: Testing %d pieces\n", num_test_blocks);
+
+		/* backup current block id, we need field for cpu algorithm */
+		savedblockid = bowl->block.id;
+
+		/* let cpu algorithm compute score for all pieces */
+		for (i = 0; i < num_test_blocks; i++) {
+			bowl->block.id = next_blocks[bowl->next_blocks_pos + i];
+			bowl_compute_cpu_dest( bowl );
+			scores[i].block = bowl->block.id;
+			scores[i].next_blocks_pos = bowl->next_blocks_pos + i;
+			scores[i].score = bowl->cpu_dest_score;
+			DPRINTF("  type = %d, score = %d\n",scores[i].block, 
+					scores[i].score);
+		}
+
+		/* bubblesort scores */
+		for ( i = 0; i < num_test_blocks - 1; i++ ) {
+			for ( j = i + 1; j < num_test_blocks; j++ ) {
+				if ( scores[j].score < scores[i].score ) {
+					tmp = scores[i];
+					scores[i] = scores[j];
+					scores[j] = tmp;
+				}
+			}
+		}
+
+		/* for now always deal worst block */
+		i = next_blocks[scores[0].next_blocks_pos];
+		next_blocks[scores[0].next_blocks_pos] = 
+			next_blocks[bowl->next_blocks_pos];
+		next_blocks[bowl->next_blocks_pos] = i;
+		DPRINTF("Dealing type %d next\n", 
+				next_blocks[bowl->next_blocks_pos]);
+
+		/* restore current block id */
+		bowl->block.id = savedblockid;
+	}
+	
+	/* set new preview from next_blocks and fill more bags if end of
+	 * buffer reached */
+	bowl->next_block_id = next_blocks[bowl->next_blocks_pos++];
+	DPRINTF("Bowl %p: Current = %d, Next = %d\n", bowl, bowl->block.id, 
+							bowl->next_block_id);
+	if ( bowl->next_blocks_pos == next_blocks_size ) {
+		DPRINTF("Need to fill new tetrominoes bags\n");
+		/* we have to keep part of buffer which other bowls 
+		 * might still need to go through */
+		min = next_blocks_size;
+		for ( i = 0; i < BOWL_COUNT; i++ )
+			if ( bowls[i] && bowls[i]->next_blocks_pos < min )
+				min = bowls[i]->next_blocks_pos;
+		num_old_blocks = next_blocks_size - min;
+			
+		/* resize buffer and save old blocks */
+		next_blocks_size = num_old_blocks + BLOCK_BAG_COUNT*BLOCK_COUNT;
+		new_next_blocks = calloc(next_blocks_size, sizeof(int));
+		if (num_old_blocks > 0) {
+			memcpy( new_next_blocks, &next_blocks[min],
+					sizeof(int) * num_old_blocks );
+			DPRINTF("Keeping %d pieces: ", num_old_blocks);
+			for (i = 0; i < num_old_blocks; i++)
+				DPRINTF("%d ",new_next_blocks[i]);
+			DPRINTF("\n");
+		}
+		free( next_blocks );
+		next_blocks = new_next_blocks;
+		
+		/* adjust position in next_blocks for all bowls */
+		for ( i = 0; i < BOWL_COUNT; i++ )
+			if ( bowls[i] )
+				bowls[i]->next_blocks_pos -= min;
+			
+		/* fill new bags */
+		fill_random_block_bags( next_blocks + num_old_blocks, 
+							BLOCK_BAG_COUNT );
+	}
+    
+	/* init rest of block structure */
+	bowl->block.x = 3;
+	bowl->block.y = -3;
+	bowl->block.sx = bowl->sx + bowl->block_size * bowl->block.x;
+	bowl->block.sy = bowl->sy + bowl->block_size * bowl->block.y;
+	bowl->block.rot_id = 0;
+	bowl->block.sw = bowl->block.sh = 4 * bowl->block_size;
+	bowl->block.cur_x = bowl->block.x * bowl->block_size;
+	bowl->block.cur_y = bowl->block.y * bowl->block_size;
+	bowl->block.check_y = (int)bowl->block.cur_y;
+	bowl_compute_help_pos( bowl );
+	bowl_compute_preview_pos( bowl );
+
+	/* if CPU is in control get destination row & other stuff */
+	if ( !bowl->controls ) {
+		/* destination */
+		bowl_compute_cpu_dest( bowl );
+		
+		/* set delay until cpu waits with dropping block */
+		delay_set( &bowl->cpu_delay, config.cpu_delay );
+		bowl->cpu_down = 0;
+		delay_set( &bowl->cpu_rot_delay, config.cpu_rot_delay );
+	}
 }
 
 /*
@@ -338,8 +366,16 @@ NOTE: py is a pixel value and x is a bowl map value.
 The tolerance value moves the checked corners into the middle
 of the brick opening a small 'window' to move though it's blocked.
 Useful when config::block_by_block is disabled.
+If position is valid, 0 is returned. Other return values:
+  POSINVAL_LEFT (out of left boundary)
+  POSINVAL_RIGHT (out of right boundary)
+  POSINVAL invalid for other reasons
 ====================================================================
 */
+#define POSVALID 0
+#define POSINVAL -1
+#define POSINVAL_LEFT -2
+#define POSINVAL_RIGHT -3
 int bowl_validate_block_pos( Bowl *bowl, int x, int py, int rot, int tol )
 {
     int i, j;
@@ -347,22 +383,30 @@ int bowl_validate_block_pos( Bowl *bowl, int x, int py, int rot, int tol )
     for ( j = 0; j < 4; j++ ) {
         for ( i = 0; i < 4; i++ ) {
             if ( block_masks[bowl->block.id].mask[rot][i][j] ) {
-                if ( x + i < 0 ) return 0;
-                if ( x + i >= bowl->w ) return 0;
-                if ( py + tile_y >= bowl->sh ) return 0;
+                if ( x + i < 0 )
+                    return POSINVAL_LEFT;
+                if ( x + i >= bowl->w ) 
+                    return POSINVAL_RIGHT;
+                if ( py + tile_y >= bowl->sh )
+                    return POSINVAL;
                 /* if it doesn't fit the screen from above it's okay */
-                if ( py + tile_y < 0 ) continue;
+                if ( py + tile_y < 0 )
+                    continue;
                 /* bowl pixel contents */
-                if ( bowl->pixel_contents[i + x][py + tile_y + tol]!= -1 ) return 0;
-                if ( bowl->pixel_contents[i + x][py + tile_y + bowl->block_size - 1 - tol] != -1 ) return 0;
+                if ( bowl->pixel_contents[i + x][py + tile_y + tol]!= -1 )
+                    return POSINVAL;
+                if ( bowl->pixel_contents[i + x][py + tile_y + bowl->block_size - 1 - tol] != -1 )
+                    return POSINVAL;
                 /* if the bowl bottom is hit it is a collision as well */
-                if ( py + tile_y + tol >= bowl->sh ) return 0;
-                if ( py + tile_y + bowl->block_size - 1 - tol >= bowl->sh ) return 0;
+                if ( py + tile_y + tol >= bowl->sh )
+                    return POSINVAL;
+                if ( py + tile_y + bowl->block_size - 1 - tol >= bowl->sh )
+                    return POSINVAL;
             }
         }
         tile_y += bowl->block_size;
     }
-    return 1;
+    return POSVALID;
 }
 
 /*
@@ -418,7 +462,7 @@ void bowl_add_tile( Bowl *bowl )
             j++;
         /* add tile and test if this hits the block if so remove and try another one */
         bowl_set_tile( bowl, i, j, 9 );
-        if ( !bowl_validate_block_pos( bowl, bowl->block.x, bowl->block.check_y, bowl->block.rot_id, 0 ) ) {
+        if ( bowl_validate_block_pos( bowl, bowl->block.x, bowl->block.check_y, bowl->block.rot_id, 0 ) < 0) {
             bowl_set_tile( bowl, i, j, -1 );
             continue;
         }
@@ -478,7 +522,7 @@ int bowl_add_line( Bowl * bowl, int wanted_holes, int *holes_pos)
         }
     }
     /* check if block position became invalid */
-    if ( !bowl_validate_block_pos( bowl, bowl->block.x, bowl->block.check_y, bowl->block.rot_id, 0 ) ) {
+    if ( bowl_validate_block_pos( bowl, bowl->block.x, bowl->block.check_y, bowl->block.rot_id, 0 ) < 0 ) {
         bowl->block.y = bowl->block.check_y / bowl->block_size - 1;
         bowl->block.check_y = bowl->block.y * bowl->block_size;
         bowl->block.cur_y = bowl->block.check_y;
@@ -526,13 +570,21 @@ void bowl_finish_game( Bowl *bowl )
     if ( !bowl->mute ) sound_play( bowl->wav_explosion );
 #endif    
     /* gain the bonus score */
-    if ( !config.preview || bowl->preview_center_sx == -1 )
+    DPRINTF("Basic score before bonuses: %d\n", (int)counter_get(bowl->score));
+    if ( !config.preview || bowl->preview_center_sx == -1 ) {
         score_mod += 0.15;
-    if ( config.gametype != 2 )
+        DPRINTF("Bonus for no preview, current score mod = %1.2g\n",score_mod);
+    }
+    if ( config.gametype != 2 && config.starting_level > 0) {
         score_mod += 0.015 * config.starting_level;
-    if ( config.expert )
-	score_mod += 1.0;
+        DPRINTF("Bonus for higher starting level, current score mod = %1.2g\n",score_mod);
+    }
+    if ( config.expert && (config.gametype == 1 || config.gametype == 2) ) {
+	score_mod += 0.5;
+        DPRINTF("Bonus for expert mode, current score mod = %1.2g\n",score_mod);
+    }
     counter_add( &bowl->score, (int)( score_mod * counter_get( bowl->score )) );
+    DPRINTF("Final score with bonuses: %d\n", (int)counter_get(bowl->score));
 }
 
 /*
@@ -555,7 +607,7 @@ void bowl_insert_block( Bowl *bowl )
   int *hole_pos = 0;
   /* move block y up so it gets to the first free place */
   py = bowl->block.y * bowl->block_size;
-  while ( !bowl_validate_block_pos( bowl, bowl->block.x, py, bowl->block.rot_id, 0 ) )
+  while ( bowl_validate_block_pos( bowl, bowl->block.x, py, bowl->block.rot_id, 0 ) < 0 )
     py -= bowl->block_size;
   bowl->block.y = py / bowl->block_size;
   /* insert and check if block is out of screen */
@@ -681,7 +733,7 @@ void bowl_insert_block( Bowl *bowl )
 	  }
     }
   /* get next block */
-  bowl_create_next_block( bowl );
+  bowl_select_next_block( bowl );
 }
 /*
 ====================================================================
@@ -986,14 +1038,7 @@ Bowl *bowl_create( int x, int y, int preview_x, int preview_y, SDL_Surface *bloc
     bowl->level = (config.gametype == 2 ) ? 0 : config.starting_level;
     bowl->stored_key = -1;
     bowl_reset_contents( bowl );
-    if ( config.same_blocks_for_all && !config.expert && 
-         config.gametype >= 3 )
-    {
-        bowl->use_same_blocks = 1;
-        bowl->next_block_id = next_blocks[bowl->next_blocks_pos++];
-    }
-    else
-        bowl->next_block_id = rand() % BLOCK_COUNT;
+    bowl->next_block_id = next_blocks[bowl->next_blocks_pos++];
     delay_set( &bowl->block_hori_delay, config.hori_delay * 12 + 63 );
     bowl->block_hori_vel = (float)bowl->block_size / bowl->block_hori_delay.limit;
     bowl->block_drop_vel = 0.8;
@@ -1001,8 +1046,9 @@ Bowl *bowl_create( int x, int y, int preview_x, int preview_y, SDL_Surface *bloc
     bowl->help_sw = bowl->help_sh = bowl->block_size * 4;
     bowl->preview_center_sx = preview_x;
     bowl->preview_center_sy = preview_y;
-    bowl_create_next_block( bowl );
-    bowl->help_alpha_change = 1.2;
+    bowl_select_next_block( bowl );
+    bowl->help_alpha_change = -0.9;
+    bowl->help_alpha = 255;
     bowl->preview_alpha_change = 0.4;
     bowl->font = load_fixed_font( "f_small_white.bmp", 32, 96, 8 );
 #ifdef SOUND
@@ -1183,7 +1229,7 @@ void bowl_show( Bowl *bowl )
         }
         if ( config.help == 2 ) { /* draw help lines */
             int i, x, y, y1, y2;
-            int red = SDL_MapRGB( sdl.screen->format, 255, 0, 0 );
+            int red = SDL_MapRGB( sdl.screen->format, 64, 192, 64 );
             /* left help line */
             x = bowl->block.sx + left_x * bowl->block_size;
             y1 = bowl->block.sy + (left_y+1) * bowl->block_size;
@@ -1238,7 +1284,7 @@ void bowl_update( Bowl *bowl, int ms, int game_over )
     int old_bottom_y = ( (int)bowl->block.cur_y + bowl->block_size - 1 ) / bowl->block_size;
     int target_x; /* target screen position within bowl */
     int hori_movement = 0;
-    int new_rot;
+    int new_rot, hori_mod, ret;
     /* SCORE */
     counter_update( &bowl->score, ms );
     if ( game_over ) return;
@@ -1263,17 +1309,38 @@ void bowl_update( Bowl *bowl, int ms, int game_over )
                 delay_force_time_out( &bowl->block_hori_delay );
                 break;
             case KEY_ROT_LEFT:
-                new_rot = bowl->block.rot_id - 1;
-                if ( new_rot < 0 ) new_rot = 3;
-                if ( bowl_validate_block_pos( bowl, bowl->block.x, bowl->block.check_y, new_rot, 0 ) )
-                    bowl->block.rot_id = new_rot;
-                bowl_compute_help_pos( bowl );
-                break;
             case KEY_ROT_RIGHT:
-                new_rot = bowl->block.rot_id + 1;
-                if ( new_rot == 4 ) new_rot = 0;
-                if ( bowl_validate_block_pos( bowl, bowl->block.x, bowl->block.check_y, new_rot, 0 ) )
-                    bowl->block.rot_id = new_rot;
+                if (bowl->stored_key == KEY_ROT_LEFT) {
+                    new_rot = bowl->block.rot_id - 1;
+                    if ( new_rot < 0 )
+                        new_rot = 3;
+                } else {
+                    new_rot = bowl->block.rot_id + 1;
+                    if ( new_rot == 4 )
+                        new_rot = 0;
+                }
+                hori_mod = 0;
+                do {
+                    ret = bowl_validate_block_pos(bowl,bowl->block.x + hori_mod, 
+                                            bowl->block.check_y, new_rot, 0 );
+                    if (ret == POSINVAL_LEFT)
+                        hori_mod++;
+                    else if (ret == POSINVAL_RIGHT)
+                        hori_mod--;
+                    else {
+                        if (ret == 0)
+                            bowl->block.rot_id = new_rot;
+                        break;
+                    }
+                } while (abs(hori_mod) < 3);
+                if (ret == 0 && hori_mod) {
+                    bowl->block.x += hori_mod;
+                    hori_movement = 1;
+                    if ( config.smooth_hori ) {
+                        bowl->block.cur_x = bowl->block.x * bowl->block_size;
+                        bowl->block.sx = (int)bowl->block.cur_x + bowl->sx;
+                    }
+                }
                 bowl_compute_help_pos( bowl );
                 break;
             case KEY_DROP:
@@ -1283,12 +1350,12 @@ void bowl_update( Bowl *bowl, int ms, int game_over )
         /* update horizontal bowl position */
         if ( delay_timed_out( &bowl->block_hori_delay, ms ) ) {
             if ( ( bowl->controls && keystate[bowl->controls->left] ) || ( !bowl->controls && bowl->cpu_dest_x < bowl->block.x ) ) 
-                if ( bowl_validate_block_pos( bowl, bowl->block.x - 1, bowl->block.check_y, bowl->block.rot_id, 2 ) ) {
+                if ( bowl_validate_block_pos( bowl, bowl->block.x - 1, bowl->block.check_y, bowl->block.rot_id, 2 ) == 0 ) {
                     bowl->block.x--;
                     hori_movement = 1;
                 }
             if ( ( bowl->controls && keystate[bowl->controls->right] ) || ( !bowl->controls && bowl->cpu_dest_x > bowl->block.x ) ) 
-                if ( bowl_validate_block_pos( bowl, bowl->block.x + 1, bowl->block.check_y, bowl->block.rot_id , 2) ) {
+                if ( bowl_validate_block_pos( bowl, bowl->block.x + 1, bowl->block.check_y, bowl->block.rot_id , 2) == 0 ) {
                     bowl->block.x++;
                     hori_movement = 1;
                 }
@@ -1344,7 +1411,7 @@ void bowl_update( Bowl *bowl, int ms, int game_over )
             /* to allow horizontal movement after the block touched
                the ground we allow moving into the next block. this shouldn't be
                seen, of course */
-            if ( !config.async_col_check || bowl_validate_block_pos( bowl, bowl->block.x, (int)bowl->block.cur_y, bowl->block.rot_id, 0) )
+            if ( !config.async_col_check || bowl_validate_block_pos( bowl, bowl->block.x, (int)bowl->block.cur_y, bowl->block.rot_id, 0) == 0 )
                 bowl->block.sy = (int)bowl->block.cur_y + bowl->sy;
             else
                 bowl->block.sy = bowl->block.y * bowl->block_size + bowl->sy;
@@ -1375,9 +1442,9 @@ void bowl_update( Bowl *bowl, int ms, int game_over )
         }
     }
     else {
-        if ( bowl->help_alpha < 0 ) {
+        if ( bowl->help_alpha < 64 ) {
             bowl->help_alpha_change = -bowl->help_alpha_change;
-            bowl->help_alpha = 0;
+            bowl->help_alpha = 64;
         }
     }
     /* PREVIEW ALPHA */
