@@ -74,12 +74,13 @@ Hint strings for the menu.
 #define HINT_SMOOTHHORI _("Horizontally move block either tile-by-tile or smooth. This is just eye-candy and doesn't effect the moving speed at all.")
 #define HINT_SMOOTHVERT _("Drop block tile-by-tile or smooth.##NOTE: While tile-by-tile allows you to move a block below a tile multiple times you'll only be able to do so one time when choosing 'smooth'!#See 'Advanced Options/Collision Check' to improve this.")
 #define HINT_HORIDEL _("The less delay you take the faster the block will horizontally move and the more sensitive the input is handled.")
-#define HINT_CONTROLS _("Each control value needs a unique key for handling.##Left/Right: horizontal movement#Rotate Left/Right: block rotation#Down: faster Dropping#Drop: INSTANT drop")
+#define HINT_CONTROLS _("Left/Right: horizontal movement#Rotate Left/Right: block rotation#Down: faster Dropping#Drop: INSTANT drop")
+#define HINT_PAUSEKEY _("Key used to pause and unpause a game.")
 #define HINT_START _("Let's get it on!!!!")
 #define HINT_NAME _("Human player names. If you play against CPU it will be named as CPU-x.")
 #define HINT_STARTLEVEL _("This is your starting level which will be ignored for game 'Figures' (you'll always start at level 0 there).##Each starting level up adds you 1.5% score in the end!")
 #define HINT_PREVIEW _("Enable/Disable block preview.##If disabled you'll gain 15% score in the end!")
-#define HINT_HELP _("Shows guiding lines or a shadow of the currently dropping block so you see where it'll hit the ground.##This option has no malus/bonus.")
+#define HINT_HELP _("Shows guiding lines or a shadow of the currently dropping block so you see where it'll hit the ground.##This option has no penalty/bonus.")
 #define HINT_MPMENU _("Some multiplayer and CPU settings.")
 #define HINT_HOLES _("A line send to your opponent's bowl will have this number of holes in it. The more holes the harder it will be to remove this line so you should choose a low value (e.g. 1 or 2) for long multiplayer games.")
 #define HINT_RANDHOLES _("This option controls how the holes in the lines are created when more than one line is sent. If the option is on, the holes will appear randomly for each line. If the option is off, all the lines that are sent in one go, will have the holes in the same spot.")
@@ -104,10 +105,10 @@ Hint strings for the menu.
                        "hits the ground giving no possibility to move the block below another "\
                        "one.)#If you set this option to 'Async' you may move the block below others but "\
                        "on the cost of slight graphical errors.")
-#define HINT_EXPERT _("In expert mode the propabilities for the next blocks are uneven to give a " \
-		       "block that is most likely difficult to fit anywhere. The game becomes " \
-		       "really hard by this so all score is doubled in the end.")
-#define HINT_SAME _("If ON all players will get exactly the same blocks. So there is no disadvantage to any player due to random blocks. This does not work in expert mode as the block selection depends on your actions there.")
+#define HINT_EXPERT _("In expert mode the most unsuitable piece of the "\
+			"current bag (a bag is a set of all 7 randomly sorted basic pieces) "\
+			"is dealt out next. This option only works for single player. "\
+			"There is a 50% score bonus in the end.")
                        
 /*
 ====================================================================
@@ -202,7 +203,8 @@ Load/delete background and create and link all menus
 */
 void manager_create()
 {
-    Item *keys[3][6];
+    Item *keys[3][6]; /* player control keys */
+    Item *pause_key; /* pause key */
     Item *item;
     int filter[SDLK_LAST]; /* key filter */
     /* constant contence of switches */
@@ -269,7 +271,6 @@ void manager_create()
     filter[SDLK_ESCAPE] = 0;
     filter[SDLK_RETURN] = 0;
     filter[SDLK_q] = 0;
-    filter[SDLK_p] = 0;
     filter[SDLK_f] = 0;
     
     /* menus are added to this list for deleting later */
@@ -360,7 +361,6 @@ void manager_create()
     menu_add( game, item_create_separator( "" ) );
     menu_add( game, item_create_link( _("Back"), HINT_, _main ) );
     /* twoplayer options */
-    menu_add( twoplayer, item_create_switch( _("Same Blocks For All:"), HINT_SAME, &config.same_blocks_for_all, _("Off"), _("On") ) );
     menu_add( twoplayer, item_create_separator( "" ) );
     menu_add( twoplayer, item_create_range( _("Holes:"), HINT_HOLES, &config.holes, 1, 9, 1 ) );
     menu_add( twoplayer, item_create_switch( _("Random Holes:"), HINT_RANDHOLES, &config.rand_holes, _("Off"), _("On") ) );
@@ -371,15 +371,8 @@ void manager_create()
     menu_add( twoplayer, item_create_range( _("CPU Drop Delay:"), HINT_CPUDROP, &config.cpu_delay, 0, 2000, 100 ) );
     menu_add( twoplayer, item_create_separator( "" ) );
     menu_add( twoplayer, item_create_link( _("Back"), HINT_, game ) );
-    /* controls */
-    menu_add( cont, item_create_link( _("Player1"), HINT_CONTROLS, cont_player1 ) );
-    menu_add( cont, item_create_link( _("Player2"), HINT_CONTROLS, cont_player2 ) );
-    menu_add( cont, item_create_link( _("Player3"), HINT_CONTROLS, cont_player3 ) );
-    menu_add( cont, item_create_separator( "" ) );
-    menu_add( cont, item_create_range( _("Horizontal Delay:"),  HINT_HORIDEL,&config.hori_delay, 0, 5, 1 ) );
-    menu_add( cont, item_create_separator( "" ) );
-    menu_add( cont, item_create_link( _("Back"), HINT_, _main ) );
     /* all keys used */
+    pause_key = item_create_key( _("Pause Key:"), HINT_PAUSEKEY, &config.pause_key, filter );
     keys[0][0] = item_create_key( _("Left:"), HINT_CONTROLS, &config.player1.controls.left, filter );
     keys[0][1] = item_create_key( _("Right:"), HINT_CONTROLS, &config.player1.controls.right, filter );
     keys[0][2] = item_create_key( _("Rotate Left:"), HINT_CONTROLS, &config.player1.controls.rot_left, filter );
@@ -406,7 +399,22 @@ void manager_create()
                 for ( j = 0; j < 6; j++ ) 
                     if ( k != i || l != j )
                         value_add_other_key( keys[k][l]->value, keys[i][j]->value );
+            /* restrict pause key */
+            value_add_other_key( keys[k][l]->value, pause_key->value);
         }
+    /* restrict controls for pause key */
+    for ( i = 0; i < 3; i++ )
+        for ( j = 0; j < 6; j++ ) 
+            value_add_other_key( pause_key->value, keys[i][j]->value );
+    /* controls */
+    menu_add( cont, item_create_link( _("Player1"), HINT_CONTROLS, cont_player1 ) );
+    menu_add( cont, item_create_link( _("Player2"), HINT_CONTROLS, cont_player2 ) );
+    menu_add( cont, item_create_link( _("Player3"), HINT_CONTROLS, cont_player3 ) );
+    menu_add( cont, item_create_separator( "" ) );
+    menu_add( cont, item_create_range( _("Horizontal Delay:"),  HINT_HORIDEL,&config.hori_delay, 0, 9, 1 ) );
+    menu_add( cont, pause_key );
+    menu_add( cont, item_create_separator( "" ) );
+    menu_add( cont, item_create_link( _("Back"), HINT_, _main ) );
     /* controls player 1 */
     for ( k = 0; k < 6; k++ )
         menu_add( cont_player1, keys[0][k] );
